@@ -1,7 +1,7 @@
 /*
  * -*- coding: utf-8; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:filetype=c:et:sw=4:ts=4:sts=4:tw=100
  * libmachista.c
- * $Id: libmachista.c 95609 2012-07-18 00:14:43Z jmr@macports.org $
+ * $Id$
  *
  * Copyright (c) 2011 The MacPorts Project
  * Copyright (c) 2011 Landon Fuller <landonf@macports.org>
@@ -30,6 +30,15 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+/* required for asprintf(3) on OS X */
+#define _DARWIN_C_SOURCE
+/* required for asprintf(3) on Linux */
+#define _GNU_SOURCE
+
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -41,6 +50,7 @@
 
 #include <err.h>
 #include <string.h>
+#include <strings.h>
 
 #ifdef __MACH__
 #include <mach-o/fat.h>
@@ -106,14 +116,15 @@ char *macho_format_dylib_version (uint32_t version) {
     return result;
 }
 
-const char *macho_get_arch_name (cpu_type_t cputype) {
 #ifdef __MACH__
+const char *macho_get_arch_name (cpu_type_t cputype) {
     const NXArchInfo *archInfo = NXGetArchInfoFromCpuType(cputype, CPU_SUBTYPE_MULTIPLE);	
     if (!archInfo) {
         return NULL;
     }
     return archInfo->name;
 #else
+const char *macho_get_arch_name (cpu_type_t cputype UNUSED) {
     return NULL;
 #endif
 }
@@ -236,8 +247,8 @@ static macho_loadcmd_t *macho_loadcmdlist_append (macho_arch_t *mat) {
 #endif
 
 /* Parse a Mach-O header */
-static int parse_macho (macho_t *mt, macho_input_t *input) {
 #ifdef __MACH__
+static int parse_macho (macho_t *mt, macho_input_t *input) {
     /* Read the file type. */
     const uint32_t *magic = macho_read(input, input->data, sizeof(uint32_t));
     if (magic == NULL)
@@ -338,15 +349,21 @@ static int parse_macho (macho_t *mt, macho_input_t *input) {
     mat->mat_arch = swap32(header->cputype);
 
     /* Parse the Mach-O load commands */
-    const struct load_command *cmd = macho_offset(input, header, header_size, sizeof(struct load_command));
-    if (cmd == NULL)
-        return MACHO_ERANGE;
     uint32_t ncmds = swap32(header->ncmds);
 
+    /* Setup to jump over the header on the first pass through instead of the previous command */
+    const struct load_command *cmd = (void *)header;
+    uint32_t cmdsize = header_size;
+
     /* Iterate over the load commands */
     for (uint32_t i = 0; i < ncmds; i++) {
+        /* Load the next command */
+        cmd = macho_offset(input, cmd, cmdsize, sizeof(struct load_command));
+        if (cmd == NULL)
+            return MACHO_ERANGE;
+
         /* Load the full command */
-        uint32_t cmdsize = swap32(cmd->cmdsize);
+        cmdsize = swap32(cmd->cmdsize);
         cmd = macho_read(input, cmd, cmdsize);
         if (cmd == NULL)
             return MACHO_ERANGE;
@@ -425,22 +442,15 @@ static int parse_macho (macho_t *mt, macho_input_t *input) {
             default:
                 break;
         }
-
-        /* Load the next command */
-        cmd = macho_offset(input, cmd, cmdsize, sizeof(struct load_command));
-        if (cmd == NULL)
-            return MACHO_ERANGE;
     }
 
     return MACHO_SUCCESS;
-#else
-    return 0;
-#endif
 }
+#endif
 
 /* Parse a (possible Mach-O) file. For a more detailed description, see the header */
-int macho_parse_file(macho_handle_t *handle, const char *filepath, const macho_t **res) {
 #ifdef __MACH__
+int macho_parse_file(macho_handle_t *handle, const char *filepath, const macho_t **res) {
     int fd;
     struct stat st;
     void *data;
@@ -501,6 +511,7 @@ int macho_parse_file(macho_handle_t *handle, const char *filepath, const macho_t
 
     return ret;
 #else
+int macho_parse_file(macho_handle_t *handle UNUSED, const char *filepath UNUSED, const macho_t **res UNUSED) {
     return 0;
 #endif
 }
@@ -530,11 +541,18 @@ void macho_destroy_handle(macho_handle_t *handle) {
 
 /* Returns string representation of the MACHO_* error code constants */
 const char *macho_strerror(int err) {
-    int num = 0;
+    int num;
+#ifdef HAVE_FLS
+    num = fls(err);
+#else
+    /* Tiger compatibility, see #42186 */
+    num = 0;
     while (err > 0) {
         err >>= 1;
         num++;
     }
+#endif
+
     static char *errors[] = {
         /* 0x00 */ "Success",
         /* 0x01 */ "Error opening or reading file",
