@@ -1,6 +1,6 @@
 /*
  * curl.c
- * $Id: curl.c 88046 2011-12-17 05:39:39Z jmr@macports.org $
+ * $Id$
  *
  * Copyright (c) 2005 Paul Guyot, The MacPorts Project.
  * All rights reserved.
@@ -51,16 +51,23 @@
 
 /*
  * Some compiled-in constants that we may wish to change later, given more
- * empirical data.  These represent "best guess" values for now.
+ * empirical data. These represent "best guess" values for now.
  */
 #define _CURL_CONNECTION_TIMEOUT	((long)(30))		/* 30 seconds */
-#define _CURL_MINIMUM_XFER_SPEED	((long)1024)		/* 1Kb/sec */
+#define _CURL_MINIMUM_XFER_SPEED	((long)1024)		/* 1KB/sec */
 #define _CURL_MINIMUM_XFER_TIMEOUT	((long)(60))		/* 1 minute */
+#define _CURL_MINIMUM_PROGRESS_INTERVAL ((double)(0.2)) /* 0.2 seconds */
 
 /* ========================================================================= **
  * Definitions
  * ========================================================================= */
-#pragma mark Definitions
+
+/* ------------------------------------------------------------------------- **
+ * Global cURL handle
+ * ------------------------------------------------------------------------- */
+/* we use a single global handle rather than creating and destroying handles to
+ * take advantage of HTTP pipelining, especially to the packages servers. */
+static CURL* theHandle = NULL;
 
 /* ------------------------------------------------------------------------- **
  * Prototypes
@@ -69,14 +76,22 @@ int SetResultFromCurlErrorCode(Tcl_Interp* interp, CURLcode inErrorCode);
 int CurlFetchCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]);
 int CurlIsNewerCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]);
 int CurlGetSizeCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]);
+int CurlPostCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]);
+
+typedef struct {
+	Tcl_Interp *interp;
+	const char *proc;
+	double prevcalltime;
+} tcl_callback_t;
+
+static int CurlProgressHandler(tcl_callback_t *callback, double dltotal, double dlnow, double ultotal, double ulnow);
+static void CurlProgressCleanup(tcl_callback_t *callback);
 
 void CurlInit(void);
 
 /* ========================================================================= **
  * Entry points
  * ========================================================================= */
-#pragma mark -
-#pragma mark Entry points
 
 /**
  * Set the result if a libcurl error occurred return TCL_ERROR.
@@ -104,7 +119,7 @@ SetResultFromCurlErrorCode(Tcl_Interp *interp, CURLcode inErrorCode)
 /**
  * curl fetch subcommand entry point.
  *
- * syntax: curl fetch [-v] [--disable-epsv] [--ignore-ssl-cert] [--remote-time] [-u userpass] [--effective-url lasturlvar] url filename
+ * syntax: curl fetch [--disable-epsv] [--ignore-ssl-cert] [--remote-time] [-u userpass] [--effective-url lasturlvar] [--progress "builtin"|callback] url filename
  *
  * @param interp		current interpreter
  * @param objc			number of parameters
@@ -114,7 +129,6 @@ int
 CurlFetchCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 {
 	int theResult = TCL_OK;
-	CURL* theHandle = NULL;
 	FILE* theFile = NULL;
 	bool performFailed = false;
 	char theErrorString[CURL_ERROR_SIZE];
@@ -126,8 +140,13 @@ CurlFetchCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 		int remotetime = 0;
 		const char* theUserPassString = NULL;
 		const char* effectiveURLVarName = NULL;
+		tcl_callback_t progressCallback = {
+			.interp = interp,
+			.proc = NULL,
+			.prevcalltime = 0.0
+		};
 		char* effectiveURL = NULL;
-		char* userAgent = PACKAGE_STRING " libcurl/" LIBCURL_VERSION;
+		char* userAgent = PACKAGE_NAME "/" PACKAGE_VERSION " libcurl/" LIBCURL_VERSION;
 		int optioncrsr;
 		int lastoption;
 		const char* theURL;
@@ -145,9 +164,7 @@ CurlFetchCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 			/* get the option */
 			const char* theOption = Tcl_GetString(objv[optioncrsr]);
 
-			if (strcmp(theOption, "-v") == 0) {
-				noprogress = 0;
-			} else if (strcmp(theOption, "--disable-epsv") == 0) {
+			if (strcmp(theOption, "--disable-epsv") == 0) {
 				useepsv = 0;
 			} else if (strcmp(theOption, "--ignore-ssl-cert") == 0) {
 				ignoresslcert = 1;
@@ -189,6 +206,19 @@ CurlFetchCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 					theResult = TCL_ERROR;
 					break;
 				}
+			} else if (strcmp(theOption, "--progress") == 0) {
+				/* check we also have the parameter */
+				if (optioncrsr < lastoption) {
+					optioncrsr++;
+					noprogress = 0;
+					progressCallback.proc = Tcl_GetString(objv[optioncrsr]);
+				} else {
+					Tcl_SetResult(interp,
+						"curl fetch: --progress option requires a parameter",
+						TCL_STATIC);
+					theResult = TCL_ERROR;
+					break;
+				}
 			} else {
 				Tcl_ResetResult(interp);
 				Tcl_AppendResult(interp, "curl fetch: unknown option ", theOption, NULL);
@@ -220,7 +250,7 @@ CurlFetchCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 		}
 
 		/* Open the file */
-		theFile = fopen( theFilePath, "w" );
+		theFile = fopen(theFilePath, "w");
 		if (theFile == NULL) {
 			Tcl_SetResult(interp, strerror(errno), TCL_VOLATILE);
 			theResult = TCL_ERROR;
@@ -228,7 +258,12 @@ CurlFetchCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 		}
 
 		/* Create the CURL handle */
-		theHandle = curl_easy_init();
+		if (theHandle == NULL) {
+			/* Re-use existing handle if theHandle isn't NULL */
+			theHandle = curl_easy_init();
+		}
+		/* If we're re-using a handle, the previous call did ensure to reset it
+		 * to the default state using curl_easy_reset(3) */
 
 		/* Setup the handle */
 		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_URL, theURL);
@@ -238,17 +273,17 @@ CurlFetchCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 		}
 
 #if LIBCURL_VERSION_NUM >= 0x071304 && LIBCURL_VERSION_NUM <= 0x071307
-        /* FTP_PROXY workaround for Snow Leopard */
-        if (strncmp(theURL, "ftp:", 4) == 0) {
-            char *ftp_proxy = getenv("FTP_PROXY");
-            if (ftp_proxy) {
-                theCurlCode = curl_easy_setopt(theHandle, CURLOPT_PROXY, ftp_proxy);
-                if (theCurlCode != CURLE_OK) {
-                    theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
-                    break;
-                }
-            }
-        }
+		/* FTP_PROXY workaround for Snow Leopard */
+		if (strncmp(theURL, "ftp:", 4) == 0) {
+			char *ftp_proxy = getenv("FTP_PROXY");
+			if (ftp_proxy) {
+				theCurlCode = curl_easy_setopt(theHandle, CURLOPT_PROXY, ftp_proxy);
+				if (theCurlCode != CURLE_OK) {
+					theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+					break;
+				}
+			}
+		}
 #endif
 
 		/* -L option */
@@ -328,6 +363,21 @@ CurlFetchCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 			break;
 		}
 
+		/* we want/don't want a custom progress function */
+		if (noprogress == 0 && strcmp(progressCallback.proc, "builtin") != 0) {
+			theCurlCode = curl_easy_setopt(theHandle, CURLOPT_PROGRESSDATA, &progressCallback);
+			if (theCurlCode != CURLE_OK) {
+				theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+				break;
+			}
+
+			theCurlCode = curl_easy_setopt(theHandle, CURLOPT_PROGRESSFUNCTION, CurlProgressHandler);
+			if (theCurlCode != CURLE_OK) {
+				theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+				break;
+			}
+		}
+
 		/* we want/don't want to use epsv */
 		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_FTP_USE_EPSV, useepsv);
 		if (theCurlCode != CURLE_OK) {
@@ -386,8 +436,13 @@ CurlFetchCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 			break;
 		}
 
+		/* signal cleanup to the progress callback */
+		if (noprogress == 0 && strcmp(progressCallback.proc, "builtin") != 0) {
+			CurlProgressCleanup(&progressCallback);
+		}
+
 		/* close the file */
-		(void) fclose( theFile );
+		(void) fclose(theFile);
 		theFile = NULL;
 
 #if LIBCURL_VERSION_NUM == 0x070d01 /* work around broken Tiger version of cURL */
@@ -404,21 +459,21 @@ CurlFetchCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 				theResult = TCL_ERROR;
 				break;
 			}
-			theFile = fopen( theFilePath, "r");
+			theFile = fopen(theFilePath, "r");
 			if (theFile == NULL) {
 				Tcl_SetResult(interp, strerror(errno), TCL_VOLATILE);
 				theResult = TCL_ERROR;
 				break;
 			}
-			if ( (p = fgets(buf, BUFSIZ, theFile)) != NULL) {
+			if ((p = fgets(buf, BUFSIZ, theFile)) != NULL) {
 				/* skip stray header escaping into output */
 				if (strncmp(p, "Last-Modified:", 14) != 0)
 					rewind(theFile);
 			}
-			while ( (size = fread(buf, 1, BUFSIZ, theFile)) > 0) {
+			while ((size = fread(buf, 1, BUFSIZ, theFile)) > 0) {
 				fwrite(buf, 1, size, fp);
 			}
-			(void) fclose( theFile );
+			(void) fclose(theFile);
 			theFile = NULL;
 			fclose(fp);
 			if (rename(tmp, theFilePath) != 0) {
@@ -449,10 +504,6 @@ CurlFetchCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 				(effectiveURL == NULL || theCurlCode != CURLE_OK) ? "" : effectiveURL,
 				0);
 		}
-
-		/* clean up */
-		curl_easy_cleanup( theHandle );
-		theHandle = NULL;
 	} while (0);
 
 	if (performFailed) {
@@ -460,11 +511,12 @@ CurlFetchCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 		theResult = TCL_ERROR;
 	}
 
+	/* reset the connection */
 	if (theHandle != NULL) {
-		curl_easy_cleanup( theHandle );
+		curl_easy_reset(theHandle);
 	}
 	if (theFile != NULL) {
-		fclose( theFile );
+		fclose(theFile);
 	}
 
 	return theResult;
@@ -510,7 +562,7 @@ CurlIsNewerCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 			}
 
 			optioncrsr++;
-                }
+		}
 
 		if (optioncrsr <= lastoption) {
 			/* something went wrong */
@@ -534,7 +586,7 @@ CurlIsNewerCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 		}
 
 		/* Open the file (dev/null) */
-		theFile = fopen( "/dev/null", "a" );
+		theFile = fopen("/dev/null", "a");
 		if (theFile == NULL) {
 			Tcl_SetResult(interp, strerror(errno), TCL_VOLATILE);
 			theResult = TCL_ERROR;
@@ -542,7 +594,12 @@ CurlIsNewerCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 		}
 
 		/* Create the CURL handle */
-		theHandle = curl_easy_init();
+		if (theHandle == NULL) {
+			/* Re-use existing handle if theHandle isn't NULL */
+			theHandle = curl_easy_init();
+		}
+		/* If we're re-using a handle, the previous call did ensure to reset it
+		 * to the default state using curl_easy_reset(3) */
 
 		/* Setup the handle */
 		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_URL, theURL);
@@ -655,7 +712,7 @@ CurlIsNewerCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 		}
 
 		/* close the file */
-		(void) fclose( theFile );
+		(void) fclose(theFile);
 		theFile = NULL;
 
 		/* check everything went fine */
@@ -675,10 +732,6 @@ CurlIsNewerCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 				break;
 			}
 
-			/* clean up */
-			curl_easy_cleanup( theHandle );
-			theHandle = NULL;
-
 			/* compare this with the date provided by user */
 			if (theModDate < -1) {
 				Tcl_SetResult(interp, "Couldn't get resource modification date", TCL_STATIC);
@@ -694,8 +747,9 @@ CurlIsNewerCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 		}
 	} while (0);
 
+	/* reset the connection */
 	if (theHandle != NULL) {
-		curl_easy_cleanup(theHandle);
+		curl_easy_reset(theHandle);
 	}
 
 	if (theFile != NULL) {
@@ -744,7 +798,7 @@ CurlGetSizeCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 			}
 
 			optioncrsr++;
-                }
+		}
 
 		if (optioncrsr <= lastoption) {
 			/* something went wrong */
@@ -762,7 +816,7 @@ CurlGetSizeCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 		theURL = Tcl_GetString(objv[objc - 1]);
 
 		/* Open the file (dev/null) */
-		theFile = fopen( "/dev/null", "a" );
+		theFile = fopen("/dev/null", "a");
 		if (theFile == NULL) {
 			Tcl_SetResult(interp, strerror(errno), TCL_VOLATILE);
 			theResult = TCL_ERROR;
@@ -770,7 +824,12 @@ CurlGetSizeCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 		}
 
 		/* Create the CURL handle */
-		theHandle = curl_easy_init();
+		if (theHandle == NULL) {
+			/* Re-use existing handle if theHandle isn't NULL */
+			theHandle = curl_easy_init();
+		}
+		/* If we're re-using a handle, the previous call did ensure to reset it
+		 * to the default state using curl_easy_reset(3) */
 
 		/* Setup the handle */
 		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_URL, theURL);
@@ -878,7 +937,7 @@ CurlGetSizeCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 		}
 
 		/* close the file */
-		(void) fclose( theFile );
+		(void) fclose(theFile);
 		theFile = NULL;
 
 		theFileSize = 0.0;
@@ -890,17 +949,259 @@ CurlGetSizeCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
 			break;
 		}
 
-		/* clean up */
-		curl_easy_cleanup( theHandle );
-		theHandle = NULL;
-
 		(void) snprintf(theSizeString, sizeof(theSizeString),
 			"%.0f", theFileSize);
 		Tcl_SetResult(interp, theSizeString, TCL_VOLATILE);
 	} while (0);
 
+	/* reset the connection */
 	if (theHandle != NULL) {
-		curl_easy_cleanup(theHandle);
+		curl_easy_reset(theHandle);
+	}
+
+	if (theFile != NULL) {
+		fclose(theFile);
+	}
+
+	return theResult;
+}
+
+/**
+ * curl post postdata url
+ *
+ * syntax: curl post [--user-agent useragentstring] [--progress "builtin"|callback] postdata url
+ *
+ * @param interp		current interpreter
+ * @param objc			number of parameters
+ * @param objv			parameters
+ */
+int
+CurlPostCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[])
+{
+	int theResult = TCL_OK;
+	CURL* theHandle = NULL;
+	FILE* theFile = NULL;
+
+	do {
+		const char* theURL;
+		const char* thePostData;
+		CURLcode theCurlCode;
+		int noprogress = 1;
+		tcl_callback_t progressCallback = {
+			.interp = interp,
+			.proc = NULL,
+			.prevcalltime = 0.0
+		};
+		char* userAgent = PACKAGE_NAME "/" PACKAGE_VERSION " libcurl/" LIBCURL_VERSION;
+		int optioncrsr;
+		int lastoption;
+
+		/* we might have options and then postdata and the url */
+		/* let's process the options first */
+
+		optioncrsr = 2;
+		lastoption = objc - 3;
+		while (optioncrsr <= lastoption) {
+			/* get the option */
+			const char* theOption = Tcl_GetString(objv[optioncrsr]);
+
+			if (strcmp(theOption, "--user-agent") == 0) {
+				/* check we also have the parameter */
+				if (optioncrsr < lastoption) {
+					optioncrsr++;
+					userAgent = Tcl_GetString(objv[optioncrsr]);
+				} else {
+					Tcl_SetResult(interp,
+						"curl post: --user-agent option requires a parameter",
+						TCL_STATIC);
+					theResult = TCL_ERROR;
+					break;
+				}
+			} else if (strcmp(theOption, "--progress") == 0) {
+				/* check we also have the parameter */
+				if (optioncrsr < lastoption) {
+					optioncrsr++;
+					noprogress = 0;
+					progressCallback.proc = Tcl_GetString(objv[optioncrsr]);
+				} else {
+					Tcl_SetResult(interp,
+						"curl post: --progress option requires a parameter",
+						TCL_STATIC);
+					theResult = TCL_ERROR;
+					break;
+				}
+			} else {
+				Tcl_ResetResult(interp);
+				Tcl_AppendResult(interp, "curl post: unknown option ", theOption, NULL);
+				theResult = TCL_ERROR;
+				break;
+			}
+
+			optioncrsr++;
+		}
+
+		if (optioncrsr <= lastoption) {
+			/* something went wrong */
+			break;
+		}
+
+		/*	first (second) parameter is -v or the url,
+			second (third) parameter is the file */
+
+		if (objc >= 4) {
+			/* Retrieve the url - it is the last parameter */
+			theURL = Tcl_GetString(objv[objc - 1]);
+
+			/* Retrieve the post data - it's before the url */
+			thePostData = Tcl_GetString(objv[objc - 2]);
+		} else {
+			Tcl_WrongNumArgs(interp, 1, objv, "post [options] postdata file");
+			theResult = TCL_ERROR;
+			break;
+		}
+
+		/* Open the file (dev/null) */
+		theFile = fopen("/dev/null", "a");
+		if (theFile == NULL) {
+			Tcl_SetResult(interp, strerror(errno), TCL_VOLATILE);
+			theResult = TCL_ERROR;
+			break;
+		}
+
+		/* Create the CURL handle */
+		if (theHandle == NULL) {
+			/* Re-use existing handle if theHandle isn't NULL */
+			theHandle = curl_easy_init();
+		}
+		/* If we're re-using a handle, the previous call did ensure to reset it
+		 * to the default state using curl_easy_reset(3) */
+
+		/* Setup the handle */
+		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_URL, theURL);
+		if (theCurlCode != CURLE_OK) {
+			theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+			break;
+		}
+
+		/* Specify the POST data */
+		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_POSTFIELDS, thePostData);
+		if (theCurlCode != CURLE_OK) {
+			theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+			break;
+		}
+
+		/* -L option */
+		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_FOLLOWLOCATION, 1);
+		if (theCurlCode != CURLE_OK) {
+			theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+			break;
+		}
+
+		/* --max-redirs option, same default as curl command line */
+		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_MAXREDIRS, 50);
+		if (theCurlCode != CURLE_OK) {
+			theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+			break;
+		}
+
+		/* echo any cookies received on a redirect */
+		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_COOKIEJAR, "/dev/null");
+		if (theCurlCode != CURLE_OK) {
+			theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+			break;
+		}
+
+		/* -f option */
+		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_FAILONERROR, 1);
+		if (theCurlCode != CURLE_OK) {
+			theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+			break;
+		}
+
+		/* -A option */
+		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_USERAGENT, userAgent);
+		if (theCurlCode != CURLE_OK) {
+			theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+			break;
+		}
+
+		/* set timeout on connections */
+		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_CONNECTTIMEOUT, _CURL_CONNECTION_TIMEOUT);
+		if (theCurlCode != CURLE_OK) {
+			theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+			break;
+		}
+
+		/* set minimum connection speed */
+		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_LOW_SPEED_LIMIT, _CURL_MINIMUM_XFER_SPEED);
+		if (theCurlCode != CURLE_OK) {
+			theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+			break;
+		}
+
+		/* set timeout interval for connections < min xfer speed */
+		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_LOW_SPEED_TIME, _CURL_MINIMUM_XFER_TIMEOUT);
+		if (theCurlCode != CURLE_OK) {
+			theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+			break;
+		}
+
+		/* write to the file */
+		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_WRITEDATA, theFile);
+		if (theCurlCode != CURLE_OK) {
+			theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+			break;
+		}
+
+		/* skip the header data */
+		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_HEADER, 0);
+		if (theCurlCode != CURLE_OK) {
+			theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+			break;
+		}
+
+		/* we want/don't want progress */
+		theCurlCode = curl_easy_setopt(theHandle, CURLOPT_NOPROGRESS, noprogress);
+		if (theCurlCode != CURLE_OK) {
+			theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+			break;
+		}
+
+		/* we want/don't want a custom progress function */
+		if (noprogress == 0 && strcmp(progressCallback.proc, "builtin") != 0) {
+			theCurlCode = curl_easy_setopt(theHandle, CURLOPT_PROGRESSDATA, &progressCallback);
+			if (theCurlCode != CURLE_OK) {
+				theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+				break;
+			}
+
+			theCurlCode = curl_easy_setopt(theHandle, CURLOPT_PROGRESSFUNCTION, CurlProgressHandler);
+			if (theCurlCode != CURLE_OK) {
+				theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+				break;
+			}
+		}
+
+		/* actually perform the POST */
+		theCurlCode = curl_easy_perform(theHandle);
+		if (theCurlCode != CURLE_OK) {
+			theResult = SetResultFromCurlErrorCode(interp, theCurlCode);
+			break;
+		}
+
+		/* signal cleanup to the progress callback */
+		if (noprogress == 0 && strcmp(progressCallback.proc, "builtin") != 0) {
+			CurlProgressCleanup(&progressCallback);
+		}
+
+		/* close the file */
+		(void) fclose(theFile);
+		theFile = NULL;
+	} while (0);
+
+	/* reset the connection */
+	if (theHandle != NULL) {
+		curl_easy_reset(theHandle);
 	}
 
 	if (theFile != NULL) {
@@ -928,11 +1229,12 @@ CurlCmd(
 	typedef enum {
 		kCurlFetch,
 		kCurlIsNewer,
-		kCurlGetSize
+		kCurlGetSize,
+		kCurlPost
 	} EOption;
 
 	static const char *options[] = {
-		"fetch", "isnewer", "getsize", NULL
+		"fetch", "isnewer", "getsize", "post", NULL
 	};
 	int theResult = TCL_OK;
 	EOption theOptionIndex;
@@ -964,6 +1266,9 @@ CurlCmd(
 		case kCurlGetSize:
 			theResult = CurlGetSizeCmd(interp, objc, objv);
 			break;
+		case kCurlPost:
+			theResult = CurlPostCmd(interp, objc, objv);
+			break;
 		}
 	}
 
@@ -979,3 +1284,139 @@ CurlInit()
 {
 	curl_global_init(CURL_GLOBAL_ALL);
 }
+
+/* ========================================================================= **
+ * Callback function
+ * ========================================================================= */
+static int CurlProgressHandler(
+		tcl_callback_t *callback,
+		double dltotal,
+		double dlnow,
+		double ultotal,
+		double ulnow)
+{
+	if (dltotal == 0.0 && ultotal == 0.0 && dlnow == 0.0 && ulnow == 0.0) {
+		/*
+		 * We have no idea whether this is an up- or download. Do nothing for now.
+		 */
+		return 0;
+	}
+
+	enum {
+		UPLOAD,
+		DOWNLOAD
+	} transferType;
+
+	double total, now, speed, curtime;
+
+	if (dltotal != 0.0 || dlnow != 0.0) {
+		/* This is a download */
+		transferType = DOWNLOAD;
+		total = dltotal;
+		now = dlnow;
+	} else {
+		/* This is an upload */
+		transferType = UPLOAD;
+		total = ultotal;
+		now = ulnow;
+	}
+
+	/* Only send updates once a second */
+	curl_easy_getinfo(theHandle, CURLINFO_TOTAL_TIME, &curtime);
+	if ((curtime - callback->prevcalltime) < _CURL_MINIMUM_PROGRESS_INTERVAL) {
+		return 0;
+	}
+
+	if (callback->prevcalltime == 0.0) {
+		/* this is the first time we're calling the callback, call start
+		 * subcommand first */
+
+		/*
+		 * Command string, a space followed "start", another space and "dl" or
+		 * "ul" plus the trailing \0.
+		 */
+		char startCommandBuffer[strlen(callback->proc) + (1 + 5) + (1 + 2) + 1];
+		int startLen = 0;
+
+		startLen = snprintf(startCommandBuffer, sizeof(startCommandBuffer), "%s start %s",
+				callback->proc, (transferType == DOWNLOAD) ? "dl" : "ul");
+		if (startLen < 0 || (size_t) startLen >= sizeof(startCommandBuffer)) {
+			/* overflow */
+			fprintf(stderr, "pextlib1.0: buffer overflow in " __FILE__ ":%d. Buffer is: %s\n", __LINE__, startCommandBuffer);
+			abort();
+		}
+
+		if (TCL_ERROR == Tcl_EvalEx(callback->interp, startCommandBuffer, startLen, TCL_EVAL_GLOBAL)) {
+			fprintf(stderr, "curl progress callback failed: %s\n", Tcl_GetStringResult(callback->interp));
+			return 1;
+		}
+	}
+
+	callback->prevcalltime = curtime;
+
+	/* Get the average speed from curl */
+	if (transferType == DOWNLOAD) {
+		curl_easy_getinfo(theHandle, CURLINFO_SPEED_DOWNLOAD, &speed);
+	} else {
+		curl_easy_getinfo(theHandle, CURLINFO_SPEED_UPLOAD, &speed);
+	}
+
+	/*
+	 * We need the command string, a space and "update", another space and "dl"
+	 * or "ul", three doubles converted to string (see comment below), plus
+	 * a space character for separation per argument, so 3 * (1 + LEN_DOUBLE)
+	 * plus one character for the null-byte.
+	 */
+	char commandBuffer[strlen(callback->proc) + (1 + 6) + (1 + 2) + 3 * (1 + 12) + 1];
+	int len = 0;
+
+	/*
+	 * Format numbers using % .6g format specifier so we can always be sure
+	 * what the total length will be: .6g tells us we're using at most
+	 * 6 significant digits; that means 6 characters, another one for
+	 * a possible decimal point, another 4 for e+XX where 00 <= XX <= 99 for
+	 * exponents, and another one for a possible sign (or " " for positive
+	 * numbers). In total, the maximum length will be 12 per double formatted.
+	 */
+	len = snprintf(commandBuffer, sizeof(commandBuffer), "%s update %s % .6g % .6g % .6g",
+			callback->proc, (transferType == DOWNLOAD) ? "dl" : "ul", total, now, speed);
+	if (len < 0 || (size_t) len >= sizeof(commandBuffer)) {
+		/* overflow */
+		fprintf(stderr, "pextlib1.0: buffer overflow in " __FILE__ ":%d. Buffer is: %s\n", __LINE__, commandBuffer);
+		abort();
+	}
+
+	/*
+	 * Execute directly rather than compiling to bytecode first - the script is
+	 * likely to change in the next call anyway.
+	 */
+	if (TCL_ERROR == Tcl_EvalEx(callback->interp, commandBuffer, len, TCL_EVAL_GLOBAL)) {
+		fprintf(stderr, "curl progress callback failed: %s\n", Tcl_GetStringResult(callback->interp));
+		return 1;
+	}
+
+	return 0;
+}
+
+static void CurlProgressCleanup(
+		tcl_callback_t *callback)
+{
+	/*
+	 * Transfer complete, signal the progress callback
+	 */
+
+	/*
+	 * Command string, a space followed "finish" plus the trailing \0.
+	 */
+	char commandBuffer[strlen(callback->proc) + (1 + 6) + 1];
+	int len = 0;
+
+	len = snprintf(commandBuffer, sizeof(commandBuffer), "%s finish", callback->proc);
+	if (len < 0 || (size_t) len >= sizeof(commandBuffer)) {
+		/* overflow */
+		fprintf(stderr, "pextlib1.0: buffer overflow in " __FILE__ ":%d. Buffer is: %s\n", __LINE__, commandBuffer);
+		abort();
+	}
+
+	Tcl_EvalEx(callback->interp, commandBuffer, len, TCL_EVAL_GLOBAL);
+}
