@@ -1,6 +1,6 @@
 /*
  * curl.h
- * $Id: curl.h 47796 2009-03-06 10:59:32Z ryandesign@macports.org $
+ * $Id$
  *
  * Copyright (c) 2005 Paul Guyot, The MacPorts Project.
  * All rights reserved.
@@ -35,12 +35,6 @@
 
 #include <tcl.h>
 
-/**
- * Init libcurl.
- * libcurl will never be cleaned (where should I plug the hook?)
- */
-int CurlInit(Tcl_Interp* interp);
-
 /**
  * A native command to use libcurl.
  *
