@@ -1,23 +1,18 @@
-#!/bin/sh
+#!@TCLSH@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:filetype=tcl:et:sw=4:ts=4:sts=4
-# Run the Tcl interpreter \
-exec @TCLSH@ "$0" "$@"
-
 # Traverse through all ports, creating an index and archiving port directories
 # if requested
-# $Id: portindex.tcl 96181 2012-08-02 14:33:15Z jmr@macports.org $
+# $Id$
 
-source [file join "@macports_tcl_dir@" macports1.0 macports_fastload.tcl]
 package require macports
 package require Pextlib
 
 # Globals
-set archive 0
 set full_reindex 0
 set stats(total) 0
 set stats(failed) 0
 set stats(skipped) 0
-array set ui_options        [list]
+array set ui_options        [list ports_no_old_index_warning 1]
 array set global_options    [list]
 array set global_variations [list]
 set port_options            [list]
@@ -29,7 +24,6 @@ mportinit ui_options global_options global_variations
 proc print_usage args {
     global argv0
     puts "Usage: $argv0 \[-adf\] \[-p plat_ver_arch\] \[-o output directory\] \[directory\]"
-    puts "-a:\tArchive port directories (for remote sites). Requires -o option"
     puts "-o:\tOutput all files to specified directory"
     puts "-d:\tOutput debugging information"
     puts "-f:\tDo a full re-index instead of updating"
@@ -37,11 +31,11 @@ proc print_usage args {
 }
 
 proc pindex {portdir} {
-    global target oldfd oldmtime newest qindex fd directory archive outdir stats full_reindex \
+    global target oldfd oldmtime newest qindex fd directory outdir stats full_reindex \
            ui_options port_options save_prefix keepkeys
 
     # try to reuse the existing entry if it's still valid
-    if {$full_reindex != "1" && $archive != "1" && [info exists qindex([string tolower [file tail $portdir]])]} {
+    if {$full_reindex != 1 && [info exists qindex([string tolower [file tail $portdir]])]} {
         try {
             set mtime [file mtime [file join $directory $portdir Portfile]]
             if {$oldmtime >= $mtime} {
@@ -101,32 +95,15 @@ proc pindex {portdir} {
         mportclose $interp
         set portinfo(portdir) $portdir
         puts "Adding port $portdir"
-        if {$archive == "1"} {
-            if {![file isdirectory [file join $outdir [file dirname $portdir]]]} {
-                if {[catch {file mkdir [file join $outdir [file dirname $portdir]]} result]} {
-                    puts stderr "$result"
-                    exit 1
-                }
-            }
-            set portinfo(portarchive) [file join [file dirname $portdir] [file tail $portdir]].tgz
-            cd [file join $directory [file dirname $portinfo(portdir)]]
-            puts "Archiving port $portinfo(name) to [file join $outdir $portinfo(portarchive)]"
-            set tar [macports::findBinary tar $macports::autoconf::tar_path]
-            set gzip [macports::findBinary gzip $macports::autoconf::gzip_path]
-            if {[catch {exec $tar -cf - [file tail $portdir] | $gzip -c >[file join $outdir $portinfo(portarchive)]} result]} {
-                puts stderr "Failed to create port archive $portinfo(portarchive): $result"
-                exit 1
-            }
-        }
 
         foreach availkey [array names portinfo] {
             # store list of subports for top-level ports only
-            if {![info exists keepkeys($availkey)] && $availkey != "subports"} {
+            if {![info exists keepkeys($availkey)] && $availkey ne "subports"} {
                 unset portinfo($availkey)
             }
         }
         set output [array get portinfo]
-        set len [expr [string length $output] + 1]
+        set len [expr {[string length $output] + 1}]
         puts $fd [list $portinfo(name) $len]
         puts $fd $output
         set mtime [file mtime [file join $directory $portdir Portfile]]
@@ -157,7 +134,7 @@ proc pindex {portdir} {
                     }
                 }
                 set output [array get portinfo]
-                set len [expr [string length $output] + 1]
+                set len [expr {[string length $output] + 1}]
                 puts $fd [list $portinfo(name) $len]
                 puts $fd $output
             }
@@ -165,7 +142,7 @@ proc pindex {portdir} {
     }
 }
 
-if {[expr $argc > 8]} {
+if {$argc > 8} {
     print_usage
     exit 1
 }
@@ -174,25 +151,23 @@ for {set i 0} {$i < $argc} {incr i} {
     set arg [lindex $argv $i]
     switch -regex -- $arg {
         {^-.+} {
-            if {$arg == "-a"} { # Turn on archiving
-                set archive 1
-            } elseif {$arg == "-d"} { # Turn on debug output
+            if {$arg eq "-d"} { # Turn on debug output
                 set ui_options(ports_debug) yes
-            } elseif {$arg == "-o"} { # Set output directory
+            } elseif {$arg eq "-o"} { # Set output directory
                 incr i
                 set outdir [file join [pwd] [lindex $argv $i]]
-            } elseif {$arg == "-p"} { # Set platform
+            } elseif {$arg eq "-p"} { # Set platform
                 incr i
                 set platlist [split [lindex $argv $i] _]
                 set os_platform [lindex $platlist 0]
                 set os_major [lindex $platlist 1]
                 set os_arch [lindex $platlist 2]
-                if {$os_platform == "macosx"} {
+                if {$os_platform eq "macosx"} {
                     lappend port_options os.subplatform $os_platform os.universal_supported yes
                     set os_platform darwin
                 }
                 lappend port_options os.platform $os_platform os.major $os_major os.arch $os_arch
-            } elseif {$arg == "-f"} { # Completely rebuild index
+            } elseif {$arg eq "-f"} { # Completely rebuild index
                 set full_reindex 1
             } else {
                 puts stderr "Unknown option: $arg"
@@ -206,12 +181,6 @@ for {set i 0} {$i < $argc} {incr i} {
     }
 }
 
-if {$archive == 1 && ![info exists outdir]} {
-    puts stderr "You must specify an output directory with -o when using the -a option"
-    print_usage
-    exit 1
-}
-
 if {![info exists directory]} {
     set directory .
 }
@@ -264,7 +233,7 @@ set save_prefix ${macports::prefix}
 foreach key {categories depends_fetch depends_extract depends_build \
              depends_lib depends_run description epoch homepage \
              long_description maintainers name platforms revision variants \
-             version portdir portarchive replaced_by license installs_libs} {
+             version portdir replaced_by license installs_libs} {
     set keepkeys($key) 1
 }
 mporttraverse pindex $directory
@@ -276,6 +245,6 @@ file rename -force $tempportindex $outpath
 file mtime $outpath $newest
 mports_generate_quickindex $outpath
 puts "\nTotal number of ports parsed:\t$stats(total)\
-      \nPorts successfully parsed:\t[expr $stats(total) - $stats(failed)]\
+      \nPorts successfully parsed:\t[expr {$stats(total) - $stats(failed)}]\
       \nPorts failed:\t\t\t$stats(failed)\
       \nUp-to-date ports skipped:\t$stats(skipped)\n"
