@@ -1,9 +1,10 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:filetype=tcl:et:sw=4:ts=4:sts=4
 # portdestroot.tcl
-# $Id: portdestroot.tcl 70564 2010-08-13 18:06:40Z jmr@macports.org $
+# $Id$
 #
-# Copyright (c) 2002 - 2003 Apple Computer, Inc.
+# Copyright (c) 2002 - 2003 Apple Inc.
 # Copyright (c) 2004 - 2005 Robert Shaw <rshaw@opendarwin.org>
+# Copyright (c) 2004-2005, 2007-2013 The MacPorts Project
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -14,7 +15,7 @@
 # 2. Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in the
 #    documentation and/or other materials provided with the distribution.
-# 3. Neither the name of Apple Computer, Inc. nor the names of its contributors
+# 3. Neither the name of Apple Inc. nor the names of its contributors
 #    may be used to endorse or promote products derived from this software
 #    without specific prior written permission.
 #
@@ -46,20 +47,21 @@ namespace eval portdestroot {
 }
 
 # define options
-options destroot.target destroot.destdir destroot.clean destroot.keepdirs destroot.umask
-options destroot.violate_mtree destroot.asroot
-options startupitem.create startupitem.requires startupitem.init
-options startupitem.name startupitem.start startupitem.stop startupitem.restart
-options startupitem.type startupitem.executable
-options startupitem.pidfile startupitem.logfile startupitem.logevents startupitem.netchange
-options startupitem.uniquename startupitem.plist startupitem.location
+options destroot.target destroot.destdir destroot.clean destroot.keepdirs destroot.umask \
+        destroot.violate_mtree destroot.asroot destroot.delete_la_files \
+        startupitem.autostart startupitem.create startupitem.executable \
+        startupitem.init startupitem.install startupitem.location \
+        startupitem.logevents startupitem.logfile startupitem.name \
+        startupitem.netchange startupitem.pidfile startupitem.plist \
+        startupitem.requires startupitem.restart startupitem.start \
+        startupitem.stop startupitem.type startupitem.uniquename
 commands destroot
 
 # Set defaults
 default destroot.asroot no
 default destroot.dir {${build.dir}}
 default destroot.cmd {${build.cmd}}
-default destroot.pre_args {${destroot.target}}
+default destroot.pre_args {[portdestroot::destroot_getargs]}
 default destroot.target install
 default destroot.post_args {${destroot.destdir}}
 default destroot.destdir {DESTDIR=${destroot}}
@@ -68,32 +70,46 @@ default destroot.umask {$system_options(destroot_umask)}
 default destroot.clean no
 default destroot.keepdirs ""
 default destroot.violate_mtree no
+default destroot.delete_la_files {${delete_la_files}}
 
-default startupitem.name        {${name}}
-default startupitem.uniquename  {org.macports.${startupitem.name}}
-default startupitem.plist       {${startupitem.uniquename}.plist}
-default startupitem.location    LaunchDaemons
+default startupitem.autostart   no
+default startupitem.executable  ""
 default startupitem.init        ""
+default startupitem.install     {$system_options(startupitem_install)}
+default startupitem.location    LaunchDaemons
+default startupitem.logevents   no
+default startupitem.logfile     ""
+default startupitem.name        {${subport}}
+default startupitem.netchange   no
+default startupitem.pidfile     ""
+default startupitem.plist       {${startupitem.uniquename}.plist}
+default startupitem.requires    ""
+default startupitem.restart     ""
 default startupitem.start       ""
 default startupitem.stop        ""
-default startupitem.restart     ""
-default startupitem.requires    ""
-default startupitem.executable  ""
 default startupitem.type        {$system_options(startupitem_type)}
-default startupitem.pidfile     ""
-default startupitem.logfile     ""
-default startupitem.logevents   no
-default startupitem.netchange   no
+default startupitem.uniquename  {org.macports.${startupitem.name}}
 
 set_ui_prefix
 
+proc portdestroot::destroot_getargs {args} {
+    if {(([option build.type] eq "default" && [option os.platform] ne "freebsd") || \
+         ([option build.type] eq "gnu")) \
+        && [regexp "^(/\\S+/|)(g|gnu|)make(\\s+.*|)$" [option destroot.cmd]]} {
+        # Print "Entering directory" lines for better log debugging
+        return "-w [option destroot.target]"
+    }
+
+    return "[option destroot.target]"
+}
+
 proc portdestroot::destroot_start {args} {
-    global UI_PREFIX prefix name porturl destroot os.platform destroot.clean portsharepath
-    global destroot.umask destroot.asroot euid egid
-    global applications_dir frameworks_dir
+    global UI_PREFIX prefix subport porturl destroot os.platform destroot.clean portsharepath \
+           destroot.umask destroot.asroot euid egid \
+           applications_dir frameworks_dir
     variable oldmask
 
-    ui_msg "$UI_PREFIX [format [msgcat::mc "Staging %s into destroot"] ${name}]"
+    ui_notice "$UI_PREFIX [format [msgcat::mc "Staging %s into destroot"] ${subport}]"
 
     # start gsoc08-privileges
     if { [getuid] == 0 && [geteuid] != 0 } {
@@ -101,8 +117,8 @@ proc portdestroot::destroot_start {args} {
         ui_debug "Can't run destroot under sudo without elevated privileges (due to mtree)."
         ui_debug "Run destroot without sudo to avoid root privileges."
         ui_debug "Going to escalate privileges back to root."
-        setegid $egid
         seteuid $euid
+        setegid $egid
         ui_debug "euid changed to: [geteuid]. egid changed to: [getegid]."
     }
 
@@ -135,9 +151,9 @@ proc portdestroot::destroot_main {args} {
 }
 
 proc portdestroot::destroot_finish {args} {
-    global UI_PREFIX destroot prefix name startupitem.create destroot.violate_mtree
-    global applications_dir frameworks_dir destroot.keepdirs
-    global os.platform os.version
+    global UI_PREFIX destroot prefix subport startupitem.create destroot.violate_mtree \
+           applications_dir frameworks_dir destroot.keepdirs destroot.delete_la_files \
+           os.platform os.version
     variable oldmask
 
     # Create startup-scripts/items
@@ -147,23 +163,55 @@ proc portdestroot::destroot_finish {args} {
     }
 
     foreach fileToDelete {share/info/dir lib/charset.alias} {
-        if [file exists "${destroot}${prefix}/${fileToDelete}"] {
+        if {[file exists "${destroot}${prefix}/${fileToDelete}"]} {
             ui_debug "Deleting stray ${fileToDelete} file."
             file delete "${destroot}${prefix}/${fileToDelete}"
         }
     }
 
+    # Prevent overlinking due to glibtool .la files: https://trac.macports.org/ticket/38010
+    ui_debug "Fixing glibtool .la files in destroot for ${subport}"
+    set la_file_list [list]
+    fs-traverse -depth fullpath ${destroot} {
+        if {[file extension $fullpath] eq ".la" && ([file type $fullpath] eq "file" || [file type $fullpath] eq "link")} {
+            if {[file type $fullpath] eq "link" && [file pathtype [file link $fullpath]] ne "relative"} {
+                # prepend $destroot to target of absolute symlinks
+                set checkpath ${destroot}[file link $fullpath]
+            } else {
+                set checkpath $fullpath
+            }
+            # Make sure it is from glibtool ... "a libtool library file" will appear in the first line
+            if {![catch {set fp [open $checkpath]}]} {
+                if {[gets $fp line] > 0 && [string first "a libtool library file" $line] != -1} {
+                    lappend la_file_list $fullpath
+                }
+            } else {
+                ui_debug "Failed to open $checkpath"
+            }
+            catch {close $fp}
+        }
+    }
+    foreach fullpath $la_file_list {
+        if {${destroot.delete_la_files}} {
+            ui_debug "Removing [file tail $fullpath]"
+            file delete -force ${fullpath}
+        } else {
+            ui_debug "Clearing dependency_libs in [file tail $fullpath]"
+            reinplace "/dependency_libs/ s/'.*'/''/" ${fullpath}
+        }
+    }
+
     # Prune empty directories in ${destroot}
     foreach path ${destroot.keepdirs} {
         if {![file isdirectory ${path}]} {
             xinstall -m 0755 -d ${path}
         }
-        if {![file exists ${path}/.turd_${name}]} {
-            xinstall -c -m 0644 /dev/null ${path}/.turd_${name}
+        if {![file exists ${path}/.turd_${subport}]} {
+            xinstall -c -m 0644 /dev/null ${path}/.turd_${subport}
         }
     }
     fs-traverse -depth dir ${destroot} {
-        if {[file type $dir] == "directory"} {
+        if {[file type $dir] eq "directory"} {
             catch {file delete $dir}
         }
     }
@@ -175,7 +223,7 @@ proc portdestroot::destroot_finish {args} {
                   alternative destroot mechanism in the Portfile."
         ui_error "Files might have been installed directly into your system,\
                   check before proceeding."
-        return -code error "Staging $name into destroot failed"
+        return -code error "Staging $subport into destroot failed"
     }
 
     # Compress all manpages with gzip (instead)
@@ -183,18 +231,18 @@ proc portdestroot::destroot_finish {args} {
     set gzip [findBinary gzip ${portutil::autoconf::gzip_path}]
     set gunzip "$gzip -d"
     set bunzip2 "[findBinary bzip2 ${portutil::autoconf::bzip2_path}] -d"
-    if {[file isdirectory ${manpath}] && [file type ${manpath}] == "directory"} {
-        ui_info "$UI_PREFIX [format [msgcat::mc "Compressing man pages for %s"] ${name}]"
+    if {[file isdirectory ${manpath}] && [file type ${manpath}] eq "directory"} {
+        ui_info "$UI_PREFIX [format [msgcat::mc "Compressing man pages for %s"] ${subport}]"
         set found 0
         set manlinks [list]
         foreach mandir [readdir "${manpath}"] {
             if {![regexp {^(cat|man)(.)$} ${mandir} match ignore manindex]} { continue }
             set mandirpath [file join ${manpath} ${mandir}]
-            if {[file isdirectory ${mandirpath}] && [file type ${mandirpath}] == "directory"} {
+            if {[file isdirectory ${mandirpath}] && [file type ${mandirpath}] eq "directory"} {
                 ui_debug "Scanning ${mandir}"
                 foreach manfile [readdir ${mandirpath}] {
                     set manfilepath [file join ${mandirpath} ${manfile}]
-                    if {[file isfile ${manfilepath}] && [file type ${manfilepath}] == "file"} {
+                    if {[file isfile ${manfilepath}] && [file type ${manfilepath}] eq "file"} {
                         if {[regexp "^(.*\[.\]${manindex}\[a-z\]*)\[.\]gz\$" ${manfile} gzfile manfile]} {
                             set found 1
                             system "cd ${manpath} && \
@@ -220,7 +268,7 @@ proc portdestroot::destroot_finish {args} {
                                 file attributes ${gzmanfilepath} -permissions $desired
                             }
                         }
-                    } elseif {[file type ${manfilepath}] == "link"} {
+                    } elseif {[file type ${manfilepath}] eq "link"} {
                         lappend manlinks [file join ${mandir} ${manfile}]
                     }
                 }
@@ -298,14 +346,14 @@ proc portdestroot::destroot_finish {args} {
                             set mtree_violation "yes"
                         }
                     }
-                } elseif {[string equal -length [expr [string length $dfile] + 1] $dfile/ $prefix]} {
+                } elseif {[string equal -length [expr {[string length $dfile] + 1}] $dfile/ $prefix]} {
                     # we've found a subpath of our prefix
                     lpush pathsToCheck $dfile
                 } else {
                     set dir_allowed no
                     # these files are (at least potentially) outside of the prefix
                     foreach dir "$applications_dir $frameworks_dir /Library/LaunchAgents /Library/LaunchDaemons /Library/StartupItems" {
-                        if {[string equal -length [expr [string length $dfile] + 1] $dfile/ $dir]} {
+                        if {[string equal -length [expr {[string length $dfile] + 1}] $dfile/ $dir]} {
                             # it's a prefix of one of the allowed paths
                             set dir_allowed yes
                             break
@@ -332,12 +380,12 @@ proc portdestroot::destroot_finish {args} {
 
         # abort here only so all violations can be observed
         if { ${mtree_violation} != "no" } {
-            ui_warn "[format [msgcat::mc "%s violates the layout of the ports-filesystems!"] [option name]]"
+            ui_warn "[format [msgcat::mc "%s violates the layout of the ports-filesystems!"] [option subport]]"
             ui_warn "Please fix or indicate this misbehavior (if it is intended), it will be an error in future releases!"
             # error "mtree violation!"
         }
     } else {
-        ui_msg "[format [msgcat::mc "Note: %s installs files outside the common directory structure."] [option name]]"
+        ui_warn "[format [msgcat::mc "%s installs files outside the common directory structure."] [option subport]]"
     }
 
     # Restore umask
