@@ -1,6 +1,6 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:filetype=tcl:et:sw=4:ts=4:sts=4
 # portdistfiles.tcl
-# $Id: portdistfiles.tcl 79597 2011-06-19 20:59:11Z jmr@macports.org $
+# $Id$
 #
 # Copyright (c) 2008-2011 The MacPorts Project
 # All rights reserved.
@@ -53,10 +53,12 @@ proc portdistfiles::distfiles_start {args} {
 }
 
 proc portdistfiles::distfiles_main {args} {
-    global UI_PREFIX master_sites checksums_array portdbpath dist_subdir all_dist_files
+    global UI_PREFIX master_sites patch_sites patchfiles checksums_array \
+           portdbpath dist_subdir all_dist_files
     
     # give up on ports that do not provide URLs
-    if {![info exists master_sites] || $master_sites == "{}"} {
+    if {(![info exists master_sites] || $master_sites eq "{}")
+        && (![info exists patchfiles] || ![info exists patch_sites] || $patch_sites eq "{}")} {
         return 0
     }
 
@@ -79,7 +81,7 @@ proc portdistfiles::distfiles_main {args} {
         ui_msg "\[$distfile\] [file join $portdbpath distfiles $dist_subdir $distfile]"
 
         # print checksums if available
-        if {$result == "yes" && [array get checksums_array $distfile] != ""} {
+        if {$result eq "yes" && [array get checksums_array $distfile] ne ""} {
             foreach {type sum} $checksums_array($distfile) {
                 ui_msg " $type: $sum"
             }
