@@ -1,6 +1,6 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
 # portfetch.tcl
-# $Id: portfetch.tcl 106614 2013-06-01 05:12:02Z jmr@macports.org $
+# $Id$
 #
 # Copyright (c) 2004 - 2012 The MacPorts Project
 # Copyright (c) 2002 - 2003 Apple Inc.
@@ -48,7 +48,7 @@ namespace eval portfetch {
 }
 
 # define options: distname master_sites
-options master_sites patch_sites extract.suffix distfiles patchfiles use_bzip2 use_lzma use_xz use_zip use_7z use_dmg dist_subdir \
+options master_sites patch_sites extract.suffix distfiles patchfiles use_bzip2 use_lzma use_xz use_zip use_7z use_lzip use_dmg dist_subdir \
     fetch.type fetch.user fetch.password fetch.use_epsv fetch.ignore_sslcert \
     master_sites.mirror_subdir patch_sites.mirror_subdir \
     bzr.url bzr.revision \
@@ -117,7 +117,6 @@ default fetch.ignore_sslcert "no"
 # Use remote timestamps
 default fetch.remote_time "no"
 
-default fallback_mirror_site "macports"
 default global_mirror_site "macports_distfiles"
 default mirror_sites.listfile {"mirror_sites.tcl"}
 default mirror_sites.listpath {"port1.0/fetch"}
@@ -128,6 +127,7 @@ option_proc use_lzma  portfetch::set_extract_type
 option_proc use_xz    portfetch::set_extract_type
 option_proc use_zip   portfetch::set_extract_type
 option_proc use_7z    portfetch::set_extract_type
+option_proc use_lzip  portfetch::set_extract_type
 option_proc use_dmg   portfetch::set_extract_type
 
 option_proc fetch.type portfetch::set_fetch_type
@@ -155,6 +155,10 @@ proc portfetch::set_extract_type {option action args} {
                 set extract.suffix .7z
                 depends_extract-append bin:7za:p7zip
             }
+            use_lzip {
+                set extract.suffix .tar.lz
+                depends_extract-append bin:lzip:lzip
+            }
             use_dmg {
                 set extract.suffix .dmg
             }
@@ -165,7 +169,7 @@ proc portfetch::set_extract_type {option action args} {
 proc portfetch::set_fetch_type {option action args} {
     global os.platform os.major
     if {[string equal ${action} "set"]} {
-        if {$args != "standard"} {
+        if {$args ne "standard"} {
             distfiles
         }
         switch $args {
@@ -183,7 +187,7 @@ proc portfetch::set_fetch_type {option action args} {
                 }
             }
             git {
-                depends_fetch-append bin:git:git-core
+                depends_fetch-append bin:git:git
             }
             hg {
                 depends_fetch-append bin:hg:mercurial
@@ -225,7 +229,7 @@ proc portfetch::checkpatchfiles {urls} {
                 set distsite [getdisttag $file]
                 set file [getdistname $file]
                 lappend all_dist_files $file
-                if {$distsite != ""} {
+                if {$distsite ne ""} {
                     lappend fetch_urls $distsite $file
                 } elseif {[info exists patch_sites]} {
                     lappend fetch_urls patch_sites $file
@@ -248,7 +252,7 @@ proc portfetch::checkdistfiles {urls} {
                 set distsite [getdisttag $file]
                 set file [getdistname $file]
                 lappend all_dist_files $file
-                if {$distsite != ""} {
+                if {$distsite ne ""} {
                     lappend fetch_urls $distsite $file
                 } else {
                     lappend fetch_urls master_sites $file
@@ -267,11 +271,11 @@ proc portfetch::get_full_mirror_sites_path {} {
 # Perform the full checksites/checkpatchfiles/checkdistfiles sequence.
 # This method is used by distcheck target.
 proc portfetch::checkfiles {urls} {
-    global global_mirror_site fallback_mirror_site
+    global global_mirror_site
     upvar $urls fetch_urls
 
-    checksites [list patch_sites [list $global_mirror_site $fallback_mirror_site PATCH_SITE_LOCAL] \
-                master_sites [list $global_mirror_site $fallback_mirror_site MASTER_SITE_LOCAL]] \
+    checksites [list patch_sites [list $global_mirror_site PATCH_SITE_LOCAL] \
+                master_sites [list $global_mirror_site MASTER_SITE_LOCAL]] \
                [get_full_mirror_sites_path]
     checkpatchfiles fetch_urls
     checkdistfiles fetch_urls
@@ -385,7 +389,7 @@ proc portfetch::svn_proxy_args {url} {
     }
     regexp {(.*://)?([[:alnum:].-]+)(:(\d+))?} $proxy_str - - proxy_host - proxy_port
     set ret "--config-option servers:global:http-proxy-host=${proxy_host}"
-    if {$proxy_port != ""} {
+    if {$proxy_port ne ""} {
         append ret " --config-option servers:global:http-proxy-port=${proxy_port}"
     }
     return $ret
@@ -478,7 +482,7 @@ proc portfetch::hgfetch {args} {
 proc portfetch::fetchfiles {args} {
     global distpath all_dist_files UI_PREFIX \
            fetch.user fetch.password fetch.use_epsv fetch.ignore_sslcert fetch.remote_time \
-           fallback_mirror_site portverbose usealtworkpath altprefix
+           portverbose usealtworkpath altprefix
     variable fetch_urls
     variable urlmap
 
@@ -496,14 +500,18 @@ proc portfetch::fetchfiles {args} {
     if {${fetch.remote_time} != "no"} {
         lappend fetch_options "--remote-time"
     }
-    if {$portverbose == "yes"} {
-        lappend fetch_options "-v"
+    if {$portverbose eq "yes"} {
+        lappend fetch_options "--progress"
+        lappend fetch_options "builtin"
+    } elseif {[llength [info commands ui_progress_download]] > 0} {
+        lappend fetch_options "--progress"
+        lappend fetch_options "ui_progress_download"
     }
     set sorted no
 
     foreach {url_var distfile} $fetch_urls {
         if {![file isfile "${distpath}/${distfile}"]} {
-            ui_info "$UI_PREFIX [format [msgcat::mc "%s doesn't seem to exist in %s"] $distfile $distpath]"
+            ui_info "$UI_PREFIX [format [msgcat::mc "%s does not exist in %s"] $distfile $distpath]"
             if {![file writable $distpath]} {
                 return -code error [format [msgcat::mc "%s must be writable"] $distpath]
             }
@@ -516,7 +524,7 @@ proc portfetch::fetchfiles {args} {
                 continue
             }
             if {!$sorted} {
-                sortsites fetch_urls [mirror_sites $fallback_mirror_site {} {} [get_full_mirror_sites_path]] master_sites
+                sortsites fetch_urls master_sites
                 set sorted yes
             }
             if {![info exists urlmap($url_var)]} {
