@@ -1,6 +1,6 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
 # portimage.tcl
-# $Id: portimage.tcl 81563 2011-08-02 03:51:10Z jmr@macports.org $
+# $Id$
 #
 # Copyright (c) 2004-2005, 2007-2011 The MacPorts Project
 # Copyright (c) 2004 Will Barton <wbb4@opendarwin.org>
@@ -66,7 +66,7 @@ variable noexec 0
 
 # takes a composite version spec rather than separate version,revision,variants
 proc activate_composite {name {v ""} {optionslist ""}} {
-    if {$v == ""} {
+    if {$v eq ""} {
         return [activate $name "" "" 0 $optionslist]
     } elseif {[registry::decode_spec $v version revision variants]} {
         return [activate $name $version $revision $variants $optionslist]
@@ -88,7 +88,7 @@ proc activate {name {version ""} {revision ""} {variants 0} {optionslist ""}} {
         set noexec $options(ports_activate_no-exec)
     }
     if {![info exists registry_open]} {
-        registry::open [file join ${macports::registry.path} registry registry.db]
+        registry::open [::file join ${macports::registry.path} registry registry.db]
         set registry_open yes
     }
     set todeactivate [list]
@@ -107,19 +107,19 @@ proc activate {name {version ""} {revision ""} {variants 0} {optionslist ""}} {
         # if another version of this port is active, deactivate it first
         set current [registry::entry installed $name]
         foreach i $current {
-            if { ![string equal $specifier "[$i version]_[$i revision][$i variants]"] } {
+            if { $specifier ne "[$i version]_[$i revision][$i variants]" } {
                 lappend todeactivate $i
             }
         }
 
         # this shouldn't be possible
-        if { ![string equal [$requested installtype] "image"] } {
+        if { [$requested installtype] ne "image" } {
             return -code error "Image error: ${name} @${specifier} not installed as an image."
         }
-        if {![file isfile $location]} {
+        if {![::file isfile $location]} {
             return -code error "Image error: Can't find image file $location"
         }
-        if { [string equal [$requested state] "installed"] } {
+        if {[$requested state] eq "installed"} {
             return -code error "Image error: ${name} @${specifier} is already active."
         }
     }
@@ -137,7 +137,7 @@ proc activate {name {version ""} {revision ""} {variants 0} {optionslist ""}} {
 
 # takes a composite version spec rather than separate version,revision,variants
 proc deactivate_composite {name {v ""} {optionslist ""}} {
-    if {$v == ""} {
+    if {$v eq ""} {
         return [deactivate $name "" "" 0 $optionslist]
     } elseif {[registry::decode_spec $v version revision variants]} {
         return [deactivate $name $version $revision $variants $optionslist]
@@ -158,11 +158,11 @@ proc deactivate {name {version ""} {revision ""} {variants 0} {optionslist ""}}
         set force 0
     }
     if {![info exists registry_open]} {
-        registry::open [file join ${macports::registry.path} registry registry.db]
+        registry::open [::file join ${macports::registry.path} registry registry.db]
         set registry_open yes
     }
 
-    if { [string equal $name ""] } {
+    if {$name eq ""} {
         throw registry::image-error "Registry error: Please specify the name of the port."
     }
     set ilist [registry::entry installed $name]
@@ -175,10 +175,10 @@ proc deactivate {name {version ""} {revision ""} {variants 0} {optionslist ""}}
     set name [$requested name]
     set specifier "[$requested version]_[$requested revision][$requested variants]"
 
-    if {$version != "" && ($version != [$requested version] ||
-        ($revision != "" && ($revision != [$requested revision] || $variants != [$requested variants])))} {
+    if {$version ne "" && ($version != [$requested version] ||
+        ($revision ne "" && ($revision != [$requested revision] || $variants != [$requested variants])))} {
         set v $version
-        if {$revision != ""} {
+        if {$revision ne ""} {
             append v _${revision}${variants}
         }
         return -code error "Active version of $name is not $v but ${specifier}."
@@ -186,11 +186,11 @@ proc deactivate {name {version ""} {revision ""} {variants 0} {optionslist ""}}
 
     ui_msg "$UI_PREFIX [format [msgcat::mc "Deactivating %s @%s"] $name $specifier]"
 
-    if { ![string equal [$requested installtype] "image"] } {
+    if { [$requested installtype] ne "image" } {
         return -code error "Image error: ${name} @${specifier} not installed as an image."
     }
     # this shouldn't be possible
-    if { [$requested state] != "installed" } {
+    if { [$requested state] ne "installed" } {
         return -code error "Image error: ${name} @${specifier} is not active."
     }
 
@@ -207,12 +207,12 @@ proc _check_registry {name version revision variants} {
 
     set searchkeys $name
     set composite_spec ""
-    if {$version != ""} {
+    if {$version ne ""} {
         lappend searchkeys $version
         set composite_spec $version
         # restriction imposed by underlying registry API (see entry.c):
         # if a revision is specified, so must variants be
-        if {$revision != ""} {
+        if {$revision ne ""} {
             lappend searchkeys $revision $variants
             append composite_spec _${revision}${variants}
         }
@@ -226,7 +226,7 @@ proc _check_registry {name version revision variants} {
             set iversion [$i version]
             set irevision [$i revision]
             set ivariants [$i variants]
-            if { [$i state] == "installed" } {
+            if { [$i state] eq "installed" } {
                 ui_msg "$UI_PREFIX [format [msgcat::mc "    %s @%s_%s%s (active)"] $iname $iversion $irevision $ivariants]"
             } else {
                 ui_msg "$UI_PREFIX [format [msgcat::mc "    %s @%s_%s%s"] $iname $iversion $irevision $ivariants]"
@@ -236,7 +236,7 @@ proc _check_registry {name version revision variants} {
     } elseif { [llength $ilist] == 1 } {
         return [lindex $ilist 0]
     }
-    if {$composite_spec != ""} {
+    if {$composite_spec ne ""} {
         set composite_spec " @${composite_spec}"
     }
     throw registry::invalid "Registry error: ${name}${composite_spec} is not installed."
@@ -249,7 +249,7 @@ proc _check_registry {name version revision variants} {
 ## @param [in] dstfile path to activate file to
 ## @return 1 if file needs to be explicitly deleted if we have to roll back, 0 otherwise
 proc _activate_file {srcfile dstfile} {
-    if {[catch {set filetype [file type $srcfile]} result]} {
+    if {[catch {set filetype [::file type $srcfile]} result]} {
         # this can happen if the archive was built on case-sensitive and we're case-insensitive
         # we know any existing dstfile is ours because we checked for conflicts earlier
         if {![catch {file type $dstfile}]} {
@@ -264,23 +264,23 @@ proc _activate_file {srcfile dstfile} {
             # Don't recursively copy directories
             ui_debug "activating directory: $dstfile"
             # Don't do anything if the directory already exists.
-            if { ![file isdirectory $dstfile] } {
-                file mkdir $dstfile
+            if { ![::file isdirectory $dstfile] } {
+                ::file mkdir $dstfile
                 # fix attributes on the directory.
                 if {[getuid] == 0} {
-                    eval file attributes {$dstfile} [file attributes $srcfile]
+                    eval ::file attributes {$dstfile} [::file attributes $srcfile]
                 } else {
                     # not root, so can't set owner/group
-                    eval file attributes {$dstfile} -permissions [file attributes $srcfile -permissions]
+                    eval ::file attributes {$dstfile} -permissions [::file attributes $srcfile -permissions]
                 }
                 # set mtime on installed element
-                file mtime $dstfile [file mtime $srcfile]
+                ::file mtime $dstfile [::file mtime $srcfile]
             }
             return 0
         }
         default {
             ui_debug "activating file: $dstfile"
-            file rename $srcfile $dstfile
+            ::file rename $srcfile $dstfile
             return 1
         }
     }
@@ -289,7 +289,8 @@ proc _activate_file {srcfile dstfile} {
 # extract an archive to a temporary location
 # returns: path to the extracted directory
 proc extract_archive_to_tmpdir {location} {
-    set extractdir [mkdtemp [file join [macports::gettmpdir] mpextractXXXXXXXX]]
+    global macports::registry.path
+    set extractdir [mkdtemp [::file dirname $location]/mpextractXXXXXXXX]
     set startpwd [pwd]
 
     try {
@@ -304,7 +305,7 @@ proc extract_archive_to_tmpdir {location} {
         set unarchive.pre_args {}
         set unarchive.args {}
         set unarchive.pipe_cmd ""
-        set unarchive.type [file tail $location]
+        set unarchive.type [::file extension $location]
         switch -regex ${unarchive.type} {
             cp(io|gz) {
                 set pax "pax"
@@ -371,7 +372,7 @@ proc extract_archive_to_tmpdir {location} {
                     throw MACPORTS "No '$tar' was found on this system!"
                 }
             }
-            xar|xpkg {
+            xar {
                 set xar "xar"
                 if {[catch {set xar [macports::findBinary $xar ${macports::autoconf::xar_path}]} errmsg] == 0} {
                     ui_debug "Using $xar"
@@ -412,7 +413,7 @@ proc extract_archive_to_tmpdir {location} {
         }
         system $cmdstring
     } catch {*} {
-        file delete -force $extractdir
+        ::file delete -force $extractdir
         throw
     } finally {
         cd $startpwd
@@ -450,7 +451,7 @@ proc _activate_contents {port {imagefiles {}} {location {}}} {
                 # To be able to install links, we test if we can lstat the file to
                 # figure out if the source file exists (file exists will return
                 # false for symlinks on files that do not exist)
-                if { [catch {file lstat $srcfile dummystatvar}] } {
+                if { [catch {::file lstat $srcfile dummystatvar}] } {
                     throw registry::image-error "Image error: Source file $srcfile does not appear to exist (cannot lstat it).  Unable to activate port [$port name]."
                 }
 
@@ -468,15 +469,15 @@ proc _activate_contents {port {imagefiles {}} {location {}}} {
                     }
                 }
 
-                if {$owner != "replaced"} {
+                if {$owner ne "replaced"} {
                     if { [string is true -strict $force] } {
                         # if we're forcing the activation, then we move any existing
                         # files to a backup file, both in the filesystem and in the
                         # registry
-                        if { ![catch {file type $file}] } {
+                        if { ![catch {::file type $file}] } {
                             set bakfile "${file}${baksuffix}"
                             ui_warn "File $file already exists.  Moving to: $bakfile."
-                            file rename -force -- $file $bakfile
+                            ::file rename -force -- $file $bakfile
                             lappend backups $file
                         }
                         if { $owner != {} } {
@@ -489,7 +490,7 @@ proc _activate_contents {port {imagefiles {}} {location {}}} {
                         # the registry
                         if { $owner != {} && $owner != $port } {
                             throw registry::image-error "Image error: $file is being used by the active [$owner name] port.  Please deactivate this port first, or use 'port -f activate [$port name]' to force the activation."
-                        } elseif { $owner == {} && ![catch {file type $file}] } {
+                        } elseif { $owner == {} && ![catch {::file type $file}] } {
                             throw registry::image-error "Image error: $file already exists and does not belong to a registered port.  Unable to activate port [$port name]. Use 'port -f activate [$port name]' to force the activation."
                         }
                     }
@@ -503,10 +504,10 @@ proc _activate_contents {port {imagefiles {}} {location {}}} {
                 # subpaths. We do need, nevertheless, all sub paths to make sure
                 # we'll set the directory attributes properly for all
                 # directories.
-                set directory [file dirname $file]
+                set directory [::file dirname $file]
                 while { [lsearch -exact $files $directory] == -1 } {
                     lappend files $directory
-                    set directory [file dirname $directory]
+                    set directory [::file dirname $directory]
                 }
 
                 # Also add the filename to the imagefile list.
@@ -554,19 +555,19 @@ proc _activate_contents {port {imagefiles {}} {location {}}} {
         # locations, then rethrow the error. Transaction rollback will take care
         # of this in the registry.
         foreach file $backups {
-            file rename -force -- "${file}${baksuffix}" $file
+            ::file rename -force -- "${file}${baksuffix}" $file
         }
         # reactivate deactivated ports
         foreach entry [array names todeactivate] {
-            if {[$entry state] == "imaged" && ($noexec || ![registry::run_target $entry activate ""])} {
+            if {[$entry state] eq "imaged" && ($noexec || ![registry::run_target $entry activate ""])} {
                 activate [$entry name] [$entry version] [$entry revision] [$entry variants] [list ports_activate_no-exec $noexec]
             }
         }
         # remove temp image dir
-        file delete -force $extracted_dir
+        ::file delete -force $extracted_dir
         throw
     }
-    file delete -force $extracted_dir
+    ::file delete -force $extracted_dir
 }
 
 # These directories should not be removed during deactivation even if they are empty.
@@ -575,16 +576,20 @@ variable precious_dirs
 array set precious_dirs { /Library/LaunchDaemons 1 /Library/LaunchAgents 1 }
 
 proc _deactivate_file {dstfile} {
-    if { [file type $dstfile] == "link" } {
+    if {[catch {::file type $dstfile} filetype]} {
+        ui_debug "$dstfile does not exist"
+        return
+    }
+    if { $filetype eq "link" } {
         ui_debug "deactivating link: $dstfile"
         file delete -- $dstfile
-    } elseif { [file isdirectory $dstfile] } {
+    } elseif { $filetype eq "directory" } {
         # 0 item means empty.
         if { [llength [readdir $dstfile]] == 0 } {
             variable precious_dirs
             if {![info exists precious_dirs($dstfile)]} {
                 ui_debug "deactivating directory: $dstfile"
-                file delete -- $dstfile
+                ::file delete -- $dstfile
             } else {
                 ui_debug "directory $dstfile does not belong to us"
             }
@@ -593,7 +598,7 @@ proc _deactivate_file {dstfile} {
         }
     } else {
         ui_debug "deactivating file: $dstfile"
-        file delete -- $dstfile
+        ::file delete -- $dstfile
     }
 }
 
@@ -601,7 +606,7 @@ proc _deactivate_contents {port imagefiles {force 0} {rollback 0}} {
     set files [list]
 
     foreach file $imagefiles {
-        if { [file exists $file] || (![catch {file type $file}] && [file type $file] == "link") } {
+        if { [::file exists $file] || (![catch {::file type $file}] && [::file type $file] eq "link") } {
             # Normalize the file path to avoid removing the intermediate
             # symlinks (remove the empty directories instead)
             # Remark: paths in the registry may be not normalized.
@@ -612,14 +617,14 @@ proc _deactivate_contents {port imagefiles {force 0} {rollback 0}} {
             # belong to any port.
             # The custom realpath proc is necessary because file normalize
             # does not resolve symlinks on OS X < 10.6
-            set directory [realpath [file dirname $file]]
-            lappend files [file join $directory [file tail $file]]
+            set directory [realpath [::file dirname $file]]
+            lappend files [::file join $directory [::file tail $file]]
 
             # Split out the filename's subpaths and add them to the image list
             # as well.
             while { [lsearch -exact $files $directory] == -1 } {
                 lappend files $directory
-                set directory [file dirname $directory]
+                set directory [::file dirname $directory]
             }
         } else {
             ui_debug "$file does not exist."
